"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .drive import Drive, DriveTypedDict
from apideck_unify.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
import pydantic
from pydantic import model_serializer
from typing import Any, Dict
from typing_extensions import Annotated, NotRequired, TypedDict


class GetDriveResponseTypedDict(TypedDict):
    r"""Drives"""

    status_code: int
    r"""HTTP Response Status Code"""
    status: str
    r"""HTTP Response Status"""
    service: str
    r"""Apideck ID of service provider"""
    resource: str
    r"""Unified API resource name"""
    operation: str
    r"""Operation performed"""
    data: DriveTypedDict
    raw: NotRequired[Nullable[Dict[str, Any]]]
    r"""Raw response from the integration when raw=true query param is provided"""


class GetDriveResponse(BaseModel):
    r"""Drives"""

    status_code: int
    r"""HTTP Response Status Code"""

    status: str
    r"""HTTP Response Status"""

    service: str
    r"""Apideck ID of service provider"""

    resource: str
    r"""Unified API resource name"""

    operation: str
    r"""Operation performed"""

    data: Drive

    raw: Annotated[OptionalNullable[Dict[str, Any]], pydantic.Field(alias="_raw")] = (
        UNSET
    )
    r"""Raw response from the integration when raw=true query param is provided"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["_raw"]
        nullable_fields = ["_raw"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from .sdkconfiguration import SDKConfiguration
from apideck_unify.apiresourcecoverage_sdk import APIResourceCoverageSDK
from apideck_unify.apiresources import APIResources
from apideck_unify.apis import Apis
from apideck_unify.connectordocs import ConnectorDocs
from apideck_unify.connectorresources import ConnectorResources
from apideck_unify.connectors import Connectors


class ConnectorSDK(BaseSDK):
    connectors: Connectors
    connector_docs: ConnectorDocs
    connector_resources: ConnectorResources
    apis: Apis
    api_resources: APIResources
    api_resource_coverage: APIResourceCoverageSDK

    def __init__(self, sdk_config: SDKConfiguration) -> None:
        BaseSDK.__init__(self, sdk_config)
        self.sdk_configuration = sdk_config
        self._init_sdks()

    def _init_sdks(self):
        self.connectors = Connectors(self.sdk_configuration)
        self.connector_docs = ConnectorDocs(self.sdk_configuration)
        self.connector_resources = ConnectorResources(self.sdk_configuration)
        self.apis = Apis(self.sdk_configuration)
        self.api_resources = APIResources(self.sdk_configuration)
        self.api_resource_coverage = APIResourceCoverageSDK(self.sdk_configuration)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from apideck_unify.types import BaseModel
from apideck_unify.utils import FieldMetadata
from datetime import datetime
from enum import Enum
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class Classification(str, Enum):
    r"""Filter by account classification."""

    ASSET = "asset"
    EQUITY = "equity"
    EXPENSE = "expense"
    LIABILITY = "liability"
    REVENUE = "revenue"
    INCOME = "income"
    OTHER_INCOME = "other_income"
    OTHER_EXPENSE = "other_expense"
    COSTS_OF_SALES = "costs_of_sales"
    OTHER = "other"


class LedgerAccountsFilterTypedDict(TypedDict):
    updated_since: NotRequired[datetime]
    classification: NotRequired[Classification]
    r"""Filter by account classification."""


class LedgerAccountsFilter(BaseModel):
    updated_since: Annotated[Optional[datetime], FieldMetadata(query=True)] = None

    classification: Annotated[Optional[Classification], FieldMetadata(query=True)] = (
        None
    )
    r"""Filter by account classification."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .filetype import FileType
from .linkedfolder import LinkedFolder, LinkedFolderTypedDict
from .owner import Owner, OwnerTypedDict
from apideck_unify.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from datetime import datetime
from pydantic import model_serializer
from typing import Any, Dict, List, Optional
from typing_extensions import NotRequired, TypedDict


class PermissionsTypedDict(TypedDict):
    r"""Permissions the current user has on this file."""

    download: NotRequired[bool]
    r"""Whether the current user can download this file."""


class Permissions(BaseModel):
    r"""Permissions the current user has on this file."""

    download: Optional[bool] = None
    r"""Whether the current user can download this file."""


class UnifiedFileTypedDict(TypedDict):
    id: str
    r"""A unique identifier for an object."""
    name: Nullable[str]
    r"""The name of the file"""
    type: Nullable[FileType]
    r"""The type of resource. Could be file, folder or url"""
    downstream_id: NotRequired[Nullable[str]]
    r"""The third-party API ID of original entity"""
    description: NotRequired[Nullable[str]]
    r"""Optional description of the file"""
    path: NotRequired[Nullable[str]]
    r"""The full path of the file or folder (includes the file name)"""
    mime_type: NotRequired[Nullable[str]]
    r"""The MIME type of the file."""
    downloadable: NotRequired[bool]
    r"""Whether the current user can download this file"""
    size: NotRequired[Nullable[int]]
    r"""The size of the file in bytes"""
    owner: NotRequired[OwnerTypedDict]
    parent_folders: NotRequired[List[LinkedFolderTypedDict]]
    r"""The parent folders of the file, starting from the root"""
    parent_folders_complete: NotRequired[bool]
    r"""Whether the list of parent folders is complete. Some connectors only return the direct parent of a file"""
    permissions: NotRequired[PermissionsTypedDict]
    r"""Permissions the current user has on this file."""
    exportable: NotRequired[bool]
    r"""Whether the current file is exportable to other file formats. This property is relevant for proprietary file formats such as Google Docs or Dropbox Paper."""
    export_formats: NotRequired[Nullable[List[str]]]
    r"""The available file formats when exporting this file."""
    custom_mappings: NotRequired[Nullable[Dict[str, Any]]]
    r"""When custom mappings are configured on the resource, the result is included here."""
    updated_by: NotRequired[Nullable[str]]
    r"""The user who last updated the object."""
    created_by: NotRequired[Nullable[str]]
    r"""The user who created the object."""
    updated_at: NotRequired[Nullable[datetime]]
    r"""The date and time when the object was last updated."""
    created_at: NotRequired[Nullable[datetime]]
    r"""The date and time when the object was created."""


class UnifiedFile(BaseModel):
    id: str
    r"""A unique identifier for an object."""

    name: Nullable[str]
    r"""The name of the file"""

    type: Nullable[FileType]
    r"""The type of resource. Could be file, folder or url"""

    downstream_id: OptionalNullable[str] = UNSET
    r"""The third-party API ID of original entity"""

    description: OptionalNullable[str] = UNSET
    r"""Optional description of the file"""

    path: OptionalNullable[str] = UNSET
    r"""The full path of the file or folder (includes the file name)"""

    mime_type: OptionalNullable[str] = UNSET
    r"""The MIME type of the file."""

    downloadable: Optional[bool] = None
    r"""Whether the current user can download this file"""

    size: OptionalNullable[int] = UNSET
    r"""The size of the file in bytes"""

    owner: Optional[Owner] = None

    parent_folders: Optional[List[LinkedFolder]] = None
    r"""The parent folders of the file, starting from the root"""

    parent_folders_complete: Optional[bool] = None
    r"""Whether the list of parent folders is complete. Some connectors only return the direct parent of a file"""

    permissions: Optional[Permissions] = None
    r"""Permissions the current user has on this file."""

    exportable: Optional[bool] = None
    r"""Whether the current file is exportable to other file formats. This property is relevant for proprietary file formats such as Google Docs or Dropbox Paper."""

    export_formats: OptionalNullable[List[str]] = UNSET
    r"""The available file formats when exporting this file."""

    custom_mappings: OptionalNullable[Dict[str, Any]] = UNSET
    r"""When custom mappings are configured on the resource, the result is included here."""

    updated_by: OptionalNullable[str] = UNSET
    r"""The user who last updated the object."""

    created_by: OptionalNullable[str] = UNSET
    r"""The user who created the object."""

    updated_at: OptionalNullable[datetime] = UNSET
    r"""The date and time when the object was last updated."""

    created_at: OptionalNullable[datetime] = UNSET
    r"""The date and time when the object was created."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "downstream_id",
            "description",
            "path",
            "mime_type",
            "downloadable",
            "size",
            "owner",
            "parent_folders",
            "parent_folders_complete",
            "permissions",
            "exportable",
            "export_formats",
            "custom_mappings",
            "updated_by",
            "created_by",
            "updated_at",
            "created_at",
        ]
        nullable_fields = [
            "downstream_id",
            "name",
            "description",
            "type",
            "path",
            "mime_type",
            "size",
            "export_formats",
            "custom_mappings",
            "updated_by",
            "created_by",
            "updated_at",
            "created_at",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .currency import Currency
from .profitandlossindicator import (
    ProfitAndLossIndicator,
    ProfitAndLossIndicatorTypedDict,
)
from .profitandlosstype import ProfitAndLossType
from apideck_unify.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from pydantic import model_serializer
from typing import Any, Dict, Optional
from typing_extensions import NotRequired, TypedDict


class IncomeTypedDict(TypedDict):
    r"""The operating income accounts"""

    total: Nullable[float]
    r"""The aggregated total of all accounts within this category."""
    records: Any
    id: NotRequired[str]
    r"""A unique identifier for an object."""
    code: NotRequired[str]
    r"""The account code of the account"""
    title: NotRequired[str]
    r"""The name of the account."""
    type: NotRequired[Nullable[ProfitAndLossType]]
    r"""The type of profit and loss"""


class Income(BaseModel):
    r"""The operating income accounts"""

    total: Nullable[float]
    r"""The aggregated total of all accounts within this category."""

    records: Any

    id: Optional[str] = None
    r"""A unique identifier for an object."""

    code: Optional[str] = None
    r"""The account code of the account"""

    title: Optional[str] = None
    r"""The name of the account."""

    type: OptionalNullable[ProfitAndLossType] = UNSET
    r"""The type of profit and loss"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["id", "code", "title", "type"]
        nullable_fields = ["type", "total"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class CostOfGoodsSoldTypedDict(TypedDict):
    r"""The cost of goods sold accounts"""

    records: Any
    id: NotRequired[str]
    r"""A unique identifier for an object."""
    code: NotRequired[str]
    r"""The account code of the account"""
    title: NotRequired[str]
    r"""The name of the account."""
    type: NotRequired[Nullable[ProfitAndLossType]]
    r"""The type of profit and loss"""
    total: NotRequired[Nullable[float]]
    r"""The aggregated total of all accounts within this category."""


class CostOfGoodsSold(BaseModel):
    r"""The cost of goods sold accounts"""

    records: Any

    id: Optional[str] = None
    r"""A unique identifier for an object."""

    code: Optional[str] = None
    r"""The account code of the account"""

    title: Optional[str] = None
    r"""The name of the account."""

    type: OptionalNullable[ProfitAndLossType] = UNSET
    r"""The type of profit and loss"""

    total: OptionalNullable[float] = UNSET
    r"""The aggregated total of all accounts within this category."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["id", "code", "title", "type", "total"]
        nullable_fields = ["type", "total"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class ExpensesModelTypedDict(TypedDict):
    r"""The operating expenses accounts"""

    total: Nullable[float]
    r"""The aggregated total of all accounts within this category."""
    records: Any
    id: NotRequired[str]
    r"""A unique identifier for an object."""
    code: NotRequired[str]
    r"""The account code of the account"""
    title: NotRequired[str]
    r"""The name of the account."""
    type: NotRequired[Nullable[ProfitAndLossType]]
    r"""The type of profit and loss"""


class ExpensesModel(BaseModel):
    r"""The operating expenses accounts"""

    total: Nullable[float]
    r"""The aggregated total of all accounts within this category."""

    records: Any

    id: Optional[str] = None
    r"""A unique identifier for an object."""

    code: Optional[str] = None
    r"""The account code of the account"""

    title: Optional[str] = None
    r"""The name of the account."""

    type: OptionalNullable[ProfitAndLossType] = UNSET
    r"""The type of profit and loss"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["id", "code", "title", "type"]
        nullable_fields = ["type", "total"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class OtherIncomeTypedDict(TypedDict):
    r"""The other income accounts"""

    records: Any
    id: NotRequired[str]
    r"""A unique identifier for an object."""
    code: NotRequired[str]
    r"""The account code of the account"""
    title: NotRequired[str]
    r"""The name of the account."""
    type: NotRequired[Nullable[ProfitAndLossType]]
    r"""The type of profit and loss"""
    total: NotRequired[Nullable[float]]
    r"""The aggregated total of all accounts within this category."""


class OtherIncome(BaseModel):
    r"""The other income accounts"""

    records: Any

    id: Optional[str] = None
    r"""A unique identifier for an object."""

    code: Optional[str] = None
    r"""The account code of the account"""

    title: Optional[str] = None
    r"""The name of the account."""

    type: OptionalNullable[ProfitAndLossType] = UNSET
    r"""The type of profit and loss"""

    total: OptionalNullable[float] = UNSET
    r"""The aggregated total of all accounts within this category."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["id", "code", "title", "type", "total"]
        nullable_fields = ["type", "total"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class OtherExpensesTypedDict(TypedDict):
    r"""The other expenses accounts"""

    records: Any
    id: NotRequired[str]
    r"""A unique identifier for an object."""
    code: NotRequired[str]
    r"""The account code of the account"""
    title: NotRequired[str]
    r"""The name of the account."""
    type: NotRequired[Nullable[ProfitAndLossType]]
    r"""The type of profit and loss"""
    total: NotRequired[Nullable[float]]
    r"""The aggregated total of all accounts within this category."""


class OtherExpenses(BaseModel):
    r"""The other expenses accounts"""

    records: Any

    id: Optional[str] = None
    r"""A unique identifier for an object."""

    code: Optional[str] = None
    r"""The account code of the account"""

    title: Optional[str] = None
    r"""The name of the account."""

    type: OptionalNullable[ProfitAndLossType] = UNSET
    r"""The type of profit and loss"""

    total: OptionalNullable[float] = UNSET
    r"""The aggregated total of all accounts within this category."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["id", "code", "title", "type", "total"]
        nullable_fields = ["type", "total"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class UncategorizedAccountsTypedDict(TypedDict):
    r"""The accounts not categorized in the other sections"""

    total: Nullable[float]
    r"""The aggregated total of all accounts within this category."""
    records: Any
    id: NotRequired[str]
    r"""A unique identifier for an object."""
    code: NotRequired[str]
    r"""The account code of the account"""
    title: NotRequired[str]
    r"""The name of the account."""
    type: NotRequired[Nullable[ProfitAndLossType]]
    r"""The type of profit and loss"""


class UncategorizedAccounts(BaseModel):
    r"""The accounts not categorized in the other sections"""

    total: Nullable[float]
    r"""The aggregated total of all accounts within this category."""

    records: Any

    id: Optional[str] = None
    r"""A unique identifier for an object."""

    code: Optional[str] = None
    r"""The account code of the account"""

    title: Optional[str] = None
    r"""The name of the account."""

    type: OptionalNullable[ProfitAndLossType] = UNSET
    r"""The type of profit and loss"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["id", "code", "title", "type"]
        nullable_fields = ["type", "total"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class ProfitAndLossTypedDict(TypedDict):
    report_name: str
    r"""The name of the report"""
    income: IncomeTypedDict
    r"""The operating income accounts"""
    expenses: ExpensesModelTypedDict
    r"""The operating expenses accounts"""
    id: NotRequired[str]
    r"""A unique identifier for an object."""
    start_date: NotRequired[str]
    r"""The start date of the report"""
    end_date: NotRequired[str]
    r"""The end date of the report"""
    currency: NotRequired[Nullable[Currency]]
    r"""Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217)."""
    cost_of_goods_sold: NotRequired[CostOfGoodsSoldTypedDict]
    r"""The cost of goods sold accounts"""
    other_income: NotRequired[OtherIncomeTypedDict]
    r"""The other income accounts"""
    other_expenses: NotRequired[OtherExpensesTypedDict]
    r"""The other expenses accounts"""
    uncategorized_accounts: NotRequired[UncategorizedAccountsTypedDict]
    r"""The accounts not categorized in the other sections"""
    gross_profit: NotRequired[ProfitAndLossIndicatorTypedDict]
    net_operating_income: NotRequired[ProfitAndLossIndicatorTypedDict]
    net_income: NotRequired[ProfitAndLossIndicatorTypedDict]
    custom_mappings: NotRequired[Nullable[Dict[str, Any]]]
    r"""When custom mappings are configured on the resource, the result is included here."""
    customer: NotRequired[str]
    r"""The customer id"""


class ProfitAndLoss(BaseModel):
    report_name: str
    r"""The name of the report"""

    income: Income
    r"""The operating income accounts"""

    expenses: ExpensesModel
    r"""The operating expenses accounts"""

    id: Optional[str] = None
    r"""A unique identifier for an object."""

    start_date: Optional[str] = None
    r"""The start date of the report"""

    end_date: Optional[str] = None
    r"""The end date of the report"""

    currency: OptionalNullable[Currency] = UNSET
    r"""Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217)."""

    cost_of_goods_sold: Optional[CostOfGoodsSold] = None
    r"""The cost of goods sold accounts"""

    other_income: Optional[OtherIncome] = None
    r"""The other income accounts"""

    other_expenses: Optional[OtherExpenses] = None
    r"""The other expenses accounts"""

    uncategorized_accounts: Optional[UncategorizedAccounts] = None
    r"""The accounts not categorized in the other sections"""

    gross_profit: Optional[ProfitAndLossIndicator] = None

    net_operating_income: Optional[ProfitAndLossIndicator] = None

    net_income: Optional[ProfitAndLossIndicator] = None

    custom_mappings: OptionalNullable[Dict[str, Any]] = UNSET
    r"""When custom mappings are configured on the resource, the result is included here."""

    customer: Optional[str] = None
    r"""The customer id"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "id",
            "start_date",
            "end_date",
            "currency",
            "cost_of_goods_sold",
            "other_income",
            "other_expenses",
            "uncategorized_accounts",
            "gross_profit",
            "net_operating_income",
            "net_income",
            "custom_mappings",
            "customer",
        ]
        nullable_fields = ["currency", "custom_mappings"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from apideck_unify.types import BaseModel
from apideck_unify.utils import FieldMetadata
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class EcommerceCustomersFilterTypedDict(TypedDict):
    email: NotRequired[str]
    r"""Customer email address to filter on"""
    phone_number: NotRequired[str]
    r"""Customer phone number to filter on"""


class EcommerceCustomersFilter(BaseModel):
    email: Annotated[Optional[str], FieldMetadata(query=True)] = None
    r"""Customer email address to filter on"""

    phone_number: Annotated[Optional[str], FieldMetadata(query=True)] = None
    r"""Customer phone number to filter on"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from apideck_unify.types import BaseModel
from enum import Enum
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class Audience(str, Enum):
    r"""Audience for the doc."""

    APPLICATION_OWNER = "application_owner"
    CONSUMER = "consumer"


class Format(str, Enum):
    r"""Format of the doc."""

    MARKDOWN = "markdown"


class ConnectorDocTypedDict(TypedDict):
    id: NotRequired[str]
    r"""A unique identifier for an object."""
    name: NotRequired[str]
    r"""Name of the doc."""
    audience: NotRequired[Audience]
    r"""Audience for the doc."""
    format_: NotRequired[Format]
    r"""Format of the doc."""
    url: NotRequired[str]
    r"""Link to fetch the content of the doc."""


class ConnectorDoc(BaseModel):
    id: Optional[str] = None
    r"""A unique identifier for an object."""

    name: Optional[str] = None
    r"""Name of the doc."""

    audience: Optional[Audience] = None
    r"""Audience for the doc."""

    format_: Annotated[Optional[Format], pydantic.Field(alias="format")] = None
    r"""Format of the doc."""

    url: Optional[str] = None
    r"""Link to fetch the content of the doc."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .assignee import Assignee, AssigneeTypedDict
from .assignee_input import AssigneeInput, AssigneeInputTypedDict
from .collectiontag import CollectionTag, CollectionTagTypedDict
from .collectiontag_input import CollectionTagInput, CollectionTagInputTypedDict
from .passthroughbody import PassThroughBody, PassThroughBodyTypedDict
from apideck_unify.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from datetime import datetime
from enum import Enum
from pydantic import model_serializer
from typing import Any, Dict, List, Optional
from typing_extensions import NotRequired, TypedDict


class Priority(str, Enum):
    r"""Priority of the ticket"""

    LOW = "low"
    NORMAL = "normal"
    HIGH = "high"
    URGENT = "urgent"


class TicketTypedDict(TypedDict):
    id: str
    r"""A unique identifier for an object."""
    parent_id: NotRequired[Nullable[str]]
    r"""The ticket's parent ID"""
    collection_id: NotRequired[Nullable[str]]
    r"""The ticket's collection ID"""
    type: NotRequired[Nullable[str]]
    r"""The ticket's type"""
    subject: NotRequired[Nullable[str]]
    r"""Subject of the ticket"""
    description: NotRequired[Nullable[str]]
    r"""The ticket's description. HTML version of description is mapped if supported by the third-party platform"""
    status: NotRequired[Nullable[str]]
    r"""The current status of the ticket. Possible values include: open, in_progress, closed, or - in cases where there is no clear mapping - the original value passed through."""
    priority: NotRequired[Nullable[Priority]]
    r"""Priority of the ticket"""
    assignees: NotRequired[List[AssigneeTypedDict]]
    updated_at: NotRequired[Nullable[datetime]]
    r"""The date and time when the object was last updated."""
    created_at: NotRequired[Nullable[datetime]]
    r"""The date and time when the object was created."""
    created_by: NotRequired[Nullable[str]]
    r"""The user who created the object."""
    due_date: NotRequired[Nullable[datetime]]
    r"""Due date of the ticket"""
    completed_at: NotRequired[Nullable[datetime]]
    r"""When the ticket was completed"""
    tags: NotRequired[List[CollectionTagTypedDict]]
    custom_mappings: NotRequired[Nullable[Dict[str, Any]]]
    r"""When custom mappings are configured on the resource, the result is included here."""
    pass_through: NotRequired[List[PassThroughBodyTypedDict]]
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""


class Ticket(BaseModel):
    id: str
    r"""A unique identifier for an object."""

    parent_id: OptionalNullable[str] = UNSET
    r"""The ticket's parent ID"""

    collection_id: OptionalNullable[str] = UNSET
    r"""The ticket's collection ID"""

    type: OptionalNullable[str] = UNSET
    r"""The ticket's type"""

    subject: OptionalNullable[str] = UNSET
    r"""Subject of the ticket"""

    description: OptionalNullable[str] = UNSET
    r"""The ticket's description. HTML version of description is mapped if supported by the third-party platform"""

    status: OptionalNullable[str] = UNSET
    r"""The current status of the ticket. Possible values include: open, in_progress, closed, or - in cases where there is no clear mapping - the original value passed through."""

    priority: OptionalNullable[Priority] = UNSET
    r"""Priority of the ticket"""

    assignees: Optional[List[Assignee]] = None

    updated_at: OptionalNullable[datetime] = UNSET
    r"""The date and time when the object was last updated."""

    created_at: OptionalNullable[datetime] = UNSET
    r"""The date and time when the object was created."""

    created_by: OptionalNullable[str] = UNSET
    r"""The user who created the object."""

    due_date: OptionalNullable[datetime] = UNSET
    r"""Due date of the ticket"""

    completed_at: OptionalNullable[datetime] = UNSET
    r"""When the ticket was completed"""

    tags: Optional[List[CollectionTag]] = None

    custom_mappings: OptionalNullable[Dict[str, Any]] = UNSET
    r"""When custom mappings are configured on the resource, the result is included here."""

    pass_through: Optional[List[PassThroughBody]] = None
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "parent_id",
            "collection_id",
            "type",
            "subject",
            "description",
            "status",
            "priority",
            "assignees",
            "updated_at",
            "created_at",
            "created_by",
            "due_date",
            "completed_at",
            "tags",
            "custom_mappings",
            "pass_through",
        ]
        nullable_fields = [
            "parent_id",
            "collection_id",
            "type",
            "subject",
            "description",
            "status",
            "priority",
            "updated_at",
            "created_at",
            "created_by",
            "due_date",
            "completed_at",
            "custom_mappings",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class TicketInputTypedDict(TypedDict):
    parent_id: NotRequired[Nullable[str]]
    r"""The ticket's parent ID"""
    type: NotRequired[Nullable[str]]
    r"""The ticket's type"""
    subject: NotRequired[Nullable[str]]
    r"""Subject of the ticket"""
    description: NotRequired[Nullable[str]]
    r"""The ticket's description. HTML version of description is mapped if supported by the third-party platform"""
    status: NotRequired[Nullable[str]]
    r"""The current status of the ticket. Possible values include: open, in_progress, closed, or - in cases where there is no clear mapping - the original value passed through."""
    priority: NotRequired[Nullable[Priority]]
    r"""Priority of the ticket"""
    assignees: NotRequired[List[AssigneeInputTypedDict]]
    due_date: NotRequired[Nullable[datetime]]
    r"""Due date of the ticket"""
    tags: NotRequired[List[CollectionTagInputTypedDict]]
    pass_through: NotRequired[List[PassThroughBodyTypedDict]]
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""


class TicketInput(BaseModel):
    parent_id: OptionalNullable[str] = UNSET
    r"""The ticket's parent ID"""

    type: OptionalNullable[str] = UNSET
    r"""The ticket's type"""

    subject: OptionalNullable[str] = UNSET
    r"""Subject of the ticket"""

    description: OptionalNullable[str] = UNSET
    r"""The ticket's description. HTML version of description is mapped if supported by the third-party platform"""

    status: OptionalNullable[str] = UNSET
    r"""The current status of the ticket. Possible values include: open, in_progress, closed, or - in cases where there is no clear mapping - the original value passed through."""

    priority: OptionalNullable[Priority] = UNSET
    r"""Priority of the ticket"""

    assignees: Optional[List[AssigneeInput]] = None

    due_date: OptionalNullable[datetime] = UNSET
    r"""Due date of the ticket"""

    tags: Optional[List[CollectionTagInput]] = None

    pass_through: Optional[List[PassThroughBody]] = None
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "parent_id",
            "type",
            "subject",
            "description",
            "status",
            "priority",
            "assignees",
            "due_date",
            "tags",
            "pass_through",
        ]
        nullable_fields = [
            "parent_id",
            "type",
            "subject",
            "description",
            "status",
            "priority",
            "due_date",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .address import Address, AddressTypedDict
from .customfield import CustomField, CustomFieldTypedDict
from .email import Email, EmailTypedDict
from .passthroughbody import PassThroughBody, PassThroughBodyTypedDict
from .phonenumber import PhoneNumber, PhoneNumberTypedDict
from apideck_unify.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from datetime import date, datetime
from enum import Enum
import pydantic
from pydantic import model_serializer
from typing import Any, Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ApplicantGender(str, Enum):
    r"""The gender represents the gender identity of a person."""

    MALE = "male"
    FEMALE = "female"
    UNISEX = "unisex"
    OTHER = "other"
    NOT_SPECIFIED = "not_specified"


class ApplicantType(str, Enum):
    r"""The type of website"""

    PRIMARY = "primary"
    SECONDARY = "secondary"
    WORK = "work"
    PERSONAL = "personal"
    OTHER = "other"


class WebsitesTypedDict(TypedDict):
    url: str
    r"""The website URL"""
    id: NotRequired[Nullable[str]]
    r"""Unique identifier for the website"""
    type: NotRequired[Nullable[ApplicantType]]
    r"""The type of website"""


class Websites(BaseModel):
    url: str
    r"""The website URL"""

    id: OptionalNullable[str] = UNSET
    r"""Unique identifier for the website"""

    type: OptionalNullable[ApplicantType] = UNSET
    r"""The type of website"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["id", "type"]
        nullable_fields = ["id", "type"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class SocialLinksTypedDict(TypedDict):
    url: str
    r"""URL of the social link, e.g. https://www.twitter.com/apideck"""
    id: NotRequired[Nullable[str]]
    r"""Unique identifier of the social link"""
    type: NotRequired[Nullable[str]]
    r"""Type of the social link, e.g. twitter"""


class SocialLinks(BaseModel):
    url: str
    r"""URL of the social link, e.g. https://www.twitter.com/apideck"""

    id: OptionalNullable[str] = UNSET
    r"""Unique identifier of the social link"""

    type: OptionalNullable[str] = UNSET
    r"""Type of the social link, e.g. twitter"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["id", "type"]
        nullable_fields = ["id", "type"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class ApplicantTypedDict(TypedDict):
    id: NotRequired[str]
    r"""A unique identifier for an object."""
    name: NotRequired[str]
    r"""The name of an applicant."""
    salutation: NotRequired[Nullable[str]]
    r"""A formal salutation for the person. For example, 'Mr', 'Mrs'"""
    first_name: NotRequired[Nullable[str]]
    r"""The first name of the person."""
    last_name: NotRequired[Nullable[str]]
    r"""The last name of the person."""
    middle_name: NotRequired[Nullable[str]]
    r"""Middle name of the person."""
    initials: NotRequired[Nullable[str]]
    r"""The initials of the person, usually derived from their first, middle, and last names."""
    birthday: NotRequired[Nullable[date]]
    r"""The date of birth of the person."""
    gender: NotRequired[Nullable[ApplicantGender]]
    r"""The gender represents the gender identity of a person."""
    social_security_number: NotRequired[Nullable[str]]
    r"""A unique identifier assigned by the government. This field is considered sensitive information and may be subject to special security and privacy restrictions."""
    type: NotRequired[str]
    cover_letter: NotRequired[str]
    job_url: NotRequired[Nullable[str]]
    photo_url: NotRequired[Nullable[str]]
    r"""The URL of the photo of a person."""
    headline: NotRequired[str]
    r"""Typically a list of previous companies where the contact has worked or schools that the contact has attended"""
    title: NotRequired[Nullable[str]]
    r"""The job title of the person."""
    emails: NotRequired[List[EmailTypedDict]]
    custom_fields: NotRequired[List[CustomFieldTypedDict]]
    phone_numbers: NotRequired[List[PhoneNumberTypedDict]]
    addresses: NotRequired[List[AddressTypedDict]]
    websites: NotRequired[List[WebsitesTypedDict]]
    social_links: NotRequired[List[SocialLinksTypedDict]]
    stage_id: NotRequired[str]
    recruiter_id: NotRequired[str]
    coordinator_id: NotRequired[str]
    application_ids: NotRequired[Nullable[List[str]]]
    applications: NotRequired[Nullable[List[str]]]
    r"""Deprecated: Use application_ids instead. Array of application IDs associated with the applicant."""
    followers: NotRequired[Nullable[List[str]]]
    sources: NotRequired[Nullable[List[str]]]
    source_id: NotRequired[str]
    confidential: NotRequired[bool]
    anonymized: NotRequired[bool]
    tags: NotRequired[Nullable[List[str]]]
    archived: NotRequired[Nullable[bool]]
    last_interaction_at: NotRequired[Nullable[datetime]]
    owner_id: NotRequired[Nullable[str]]
    sourced_by: NotRequired[Nullable[str]]
    cv_url: NotRequired[str]
    record_url: NotRequired[Nullable[str]]
    rejected_at: NotRequired[Nullable[datetime]]
    custom_mappings: NotRequired[Nullable[Dict[str, Any]]]
    r"""When custom mappings are configured on the resource, the result is included here."""
    deleted: NotRequired[Nullable[bool]]
    r"""Flag to indicate if the object is deleted."""
    deleted_by: NotRequired[Nullable[str]]
    r"""The user who deleted the object."""
    deleted_at: NotRequired[Nullable[datetime]]
    r"""The time at which the object was deleted."""
    updated_by: NotRequired[Nullable[str]]
    r"""The user who last updated the object."""
    created_by: NotRequired[Nullable[str]]
    r"""The user who created the object."""
    updated_at: NotRequired[Nullable[datetime]]
    r"""The date and time when the object was last updated."""
    created_at: NotRequired[Nullable[datetime]]
    r"""The date and time when the object was created."""
    pass_through: NotRequired[List[PassThroughBodyTypedDict]]
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""


class Applicant(BaseModel):
    id: Optional[str] = None
    r"""A unique identifier for an object."""

    name: Optional[str] = None
    r"""The name of an applicant."""

    salutation: OptionalNullable[str] = UNSET
    r"""A formal salutation for the person. For example, 'Mr', 'Mrs'"""

    first_name: OptionalNullable[str] = UNSET
    r"""The first name of the person."""

    last_name: OptionalNullable[str] = UNSET
    r"""The last name of the person."""

    middle_name: OptionalNullable[str] = UNSET
    r"""Middle name of the person."""

    initials: OptionalNullable[str] = UNSET
    r"""The initials of the person, usually derived from their first, middle, and last names."""

    birthday: OptionalNullable[date] = UNSET
    r"""The date of birth of the person."""

    gender: OptionalNullable[ApplicantGender] = UNSET
    r"""The gender represents the gender identity of a person."""

    social_security_number: OptionalNullable[str] = UNSET
    r"""A unique identifier assigned by the government. This field is considered sensitive information and may be subject to special security and privacy restrictions."""

    type: Optional[str] = None

    cover_letter: Optional[str] = None

    job_url: OptionalNullable[str] = UNSET

    photo_url: OptionalNullable[str] = UNSET
    r"""The URL of the photo of a person."""

    headline: Optional[str] = None
    r"""Typically a list of previous companies where the contact has worked or schools that the contact has attended"""

    title: OptionalNullable[str] = UNSET
    r"""The job title of the person."""

    emails: Optional[List[Email]] = None

    custom_fields: Optional[List[CustomField]] = None

    phone_numbers: Optional[List[PhoneNumber]] = None

    addresses: Optional[List[Address]] = None

    websites: Optional[List[Websites]] = None

    social_links: Optional[List[SocialLinks]] = None

    stage_id: Optional[str] = None

    recruiter_id: Optional[str] = None

    coordinator_id: Optional[str] = None

    application_ids: OptionalNullable[List[str]] = UNSET

    applications: Annotated[
        OptionalNullable[List[str]],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
        ),
    ] = UNSET
    r"""Deprecated: Use application_ids instead. Array of application IDs associated with the applicant."""

    followers: OptionalNullable[List[str]] = UNSET

    sources: OptionalNullable[List[str]] = UNSET

    source_id: Optional[str] = None

    confidential: Optional[bool] = None

    anonymized: Optional[bool] = None

    tags: OptionalNullable[List[str]] = UNSET

    archived: OptionalNullable[bool] = UNSET

    last_interaction_at: OptionalNullable[datetime] = UNSET

    owner_id: OptionalNullable[str] = UNSET

    sourced_by: OptionalNullable[str] = UNSET

    cv_url: Optional[str] = None

    record_url: OptionalNullable[str] = UNSET

    rejected_at: OptionalNullable[datetime] = UNSET

    custom_mappings: OptionalNullable[Dict[str, Any]] = UNSET
    r"""When custom mappings are configured on the resource, the result is included here."""

    deleted: OptionalNullable[bool] = UNSET
    r"""Flag to indicate if the object is deleted."""

    deleted_by: OptionalNullable[str] = UNSET
    r"""The user who deleted the object."""

    deleted_at: OptionalNullable[datetime] = UNSET
    r"""The time at which the object was deleted."""

    updated_by: OptionalNullable[str] = UNSET
    r"""The user who last updated the object."""

    created_by: OptionalNullable[str] = UNSET
    r"""The user who created the object."""

    updated_at: OptionalNullable[datetime] = UNSET
    r"""The date and time when the object was last updated."""

    created_at: OptionalNullable[datetime] = UNSET
    r"""The date and time when the object was created."""

    pass_through: Optional[List[PassThroughBody]] = None
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "id",
            "name",
            "salutation",
            "first_name",
            "last_name",
            "middle_name",
            "initials",
            "birthday",
            "gender",
            "social_security_number",
            "type",
            "cover_letter",
            "job_url",
            "photo_url",
            "headline",
            "title",
            "emails",
            "custom_fields",
            "phone_numbers",
            "addresses",
            "websites",
            "social_links",
            "stage_id",
            "recruiter_id",
            "coordinator_id",
            "application_ids",
            "applications",
            "followers",
            "sources",
            "source_id",
            "confidential",
            "anonymized",
            "tags",
            "archived",
            "last_interaction_at",
            "owner_id",
            "sourced_by",
            "cv_url",
            "record_url",
            "rejected_at",
            "custom_mappings",
            "deleted",
            "deleted_by",
            "deleted_at",
            "updated_by",
            "created_by",
            "updated_at",
            "created_at",
            "pass_through",
        ]
        nullable_fields = [
            "salutation",
            "first_name",
            "last_name",
            "middle_name",
            "initials",
            "birthday",
            "gender",
            "social_security_number",
            "job_url",
            "photo_url",
            "title",
            "application_ids",
            "applications",
            "followers",
            "sources",
            "tags",
            "archived",
            "last_interaction_at",
            "owner_id",
            "sourced_by",
            "record_url",
            "rejected_at",
            "custom_mappings",
            "deleted",
            "deleted_by",
            "deleted_at",
            "updated_by",
            "created_by",
            "updated_at",
            "created_at",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class ApplicantInputTypedDict(TypedDict):
    name: NotRequired[str]
    r"""The name of an applicant."""
    salutation: NotRequired[Nullable[str]]
    r"""A formal salutation for the person. For example, 'Mr', 'Mrs'"""
    first_name: NotRequired[Nullable[str]]
    r"""The first name of the person."""
    last_name: NotRequired[Nullable[str]]
    r"""The last name of the person."""
    middle_name: NotRequired[Nullable[str]]
    r"""Middle name of the person."""
    initials: NotRequired[Nullable[str]]
    r"""The initials of the person, usually derived from their first, middle, and last names."""
    birthday: NotRequired[Nullable[date]]
    r"""The date of birth of the person."""
    gender: NotRequired[Nullable[ApplicantGender]]
    r"""The gender represents the gender identity of a person."""
    social_security_number: NotRequired[Nullable[str]]
    r"""A unique identifier assigned by the government. This field is considered sensitive information and may be subject to special security and privacy restrictions."""
    type: NotRequired[str]
    cover_letter: NotRequired[str]
    photo_url: NotRequired[Nullable[str]]
    r"""The URL of the photo of a person."""
    headline: NotRequired[str]
    r"""Typically a list of previous companies where the contact has worked or schools that the contact has attended"""
    title: NotRequired[Nullable[str]]
    r"""The job title of the person."""
    emails: NotRequired[List[EmailTypedDict]]
    custom_fields: NotRequired[List[CustomFieldTypedDict]]
    phone_numbers: NotRequired[List[PhoneNumberTypedDict]]
    addresses: NotRequired[List[AddressTypedDict]]
    websites: NotRequired[List[WebsitesTypedDict]]
    social_links: NotRequired[List[SocialLinksTypedDict]]
    stage_id: NotRequired[str]
    recruiter_id: NotRequired[str]
    coordinator_id: NotRequired[str]
    application_ids: NotRequired[Nullable[List[str]]]
    applications: NotRequired[Nullable[List[str]]]
    r"""Deprecated: Use application_ids instead. Array of application IDs associated with the applicant."""
    followers: NotRequired[Nullable[List[str]]]
    sources: NotRequired[Nullable[List[str]]]
    confidential: NotRequired[bool]
    anonymized: NotRequired[bool]
    tags: NotRequired[Nullable[List[str]]]
    archived: NotRequired[Nullable[bool]]
    owner_id: NotRequired[Nullable[str]]
    record_url: NotRequired[Nullable[str]]
    deleted: NotRequired[Nullable[bool]]
    r"""Flag to indicate if the object is deleted."""
    pass_through: NotRequired[List[PassThroughBodyTypedDict]]
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""


class ApplicantInput(BaseModel):
    name: Optional[str] = None
    r"""The name of an applicant."""

    salutation: OptionalNullable[str] = UNSET
    r"""A formal salutation for the person. For example, 'Mr', 'Mrs'"""

    first_name: OptionalNullable[str] = UNSET
    r"""The first name of the person."""

    last_name: OptionalNullable[str] = UNSET
    r"""The last name of the person."""

    middle_name: OptionalNullable[str] = UNSET
    r"""Middle name of the person."""

    initials: OptionalNullable[str] = UNSET
    r"""The initials of the person, usually derived from their first, middle, and last names."""

    birthday: OptionalNullable[date] = UNSET
    r"""The date of birth of the person."""

    gender: OptionalNullable[ApplicantGender] = UNSET
    r"""The gender represents the gender identity of a person."""

    social_security_number: OptionalNullable[str] = UNSET
    r"""A unique identifier assigned by the government. This field is considered sensitive information and may be subject to special security and privacy restrictions."""

    type: Optional[str] = None

    cover_letter: Optional[str] = None

    photo_url: OptionalNullable[str] = UNSET
    r"""The URL of the photo of a person."""

    headline: Optional[str] = None
    r"""Typically a list of previous companies where the contact has worked or schools that the contact has attended"""

    title: OptionalNullable[str] = UNSET
    r"""The job title of the person."""

    emails: Optional[List[Email]] = None

    custom_fields: Optional[List[CustomField]] = None

    phone_numbers: Optional[List[PhoneNumber]] = None

    addresses: Optional[List[Address]] = None

    websites: Optional[List[Websites]] = None

    social_links: Optional[List[SocialLinks]] = None

    stage_id: Optional[str] = None

    recruiter_id: Optional[str] = None

    coordinator_id: Optional[str] = None

    application_ids: OptionalNullable[List[str]] = UNSET

    applications: Annotated[
        OptionalNullable[List[str]],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
        ),
    ] = UNSET
    r"""Deprecated: Use application_ids instead. Array of application IDs associated with the applicant."""

    followers: OptionalNullable[List[str]] = UNSET

    sources: OptionalNullable[List[str]] = UNSET

    confidential: Optional[bool] = None

    anonymized: Optional[bool] = None

    tags: OptionalNullable[List[str]] = UNSET

    archived: OptionalNullable[bool] = UNSET

    owner_id: OptionalNullable[str] = UNSET

    record_url: OptionalNullable[str] = UNSET

    deleted: OptionalNullable[bool] = UNSET
    r"""Flag to indicate if the object is deleted."""

    pass_through: Optional[List[PassThroughBody]] = None
    r"""The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "name",
            "salutation",
            "first_name",
            "last_name",
            "middle_name",
            "initials",
            "birthday",
            "gender",
            "social_security_number",
            "type",
            "cover_letter",
            "photo_url",
            "headline",
            "title",
            "emails",
            "custom_fields",
            "phone_numbers",
            "addresses",
            "websites",
            "social_links",
            "stage_id",
            "recruiter_id",
            "coordinator_id",
            "application_ids",
            "applications",
            "followers",
            "sources",
            "confidential",
            "anonymized",
            "tags",
            "archived",
            "owner_id",
            "record_url",
            "deleted",
            "pass_through",
        ]
        nullable_fields = [
            "salutation",
            "first_name",
            "last_name",
            "middle_name",
            "initials",
            "birthday",
            "gender",
            "social_security_number",
            "photo_url",
            "title",
            "application_ids",
            "applications",
            "followers",
            "sources",
            "tags",
            "archived",
            "owner_id",
            "record_url",
            "deleted",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from apideck_unify.types import BaseModel
from apideck_unify.utils import FieldMetadata
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class AccountingDepartmentsFilterTypedDict(TypedDict):
    subsidiary: NotRequired[str]
    r"""Id of the subsidiary to search for"""


class AccountingDepartmentsFilter(BaseModel):
    subsidiary: Annotated[Optional[str], FieldMetadata(query=True)] = None
    r"""Id of the subsidiary to search for"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

import httpx
from typing import Optional


class ApideckError(Exception):
    """The base class for all HTTP error responses."""

    message: str
    status_code: int
    body: str
    headers: httpx.Headers
    raw_response: httpx.Response

    def __init__(
        self, message: str, raw_response: httpx.Response, body: Optional[str] = None
    ):
        self.message = message
        self.status_code = raw_response.status_code
        self.body = body if body is not None else raw_response.text
        self.headers = raw_response.headers
        self.raw_response = raw_response

    def __str__(self):
        return self.message



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from apideck_unify import models, utils
from apideck_unify._hooks import HookContext
from apideck_unify.types import Nullable, OptionalNullable, UNSET
from apideck_unify.utils import get_security_from_env
from apideck_unify.utils.unmarshal_json_response import unmarshal_json_response
from datetime import date
from jsonpath import JSONPath
from typing import Any, Dict, List, Mapping, Optional, Union


class Quotes(BaseSDK):
    def list(
        self,
        *,
        raw: Optional[bool] = False,
        consumer_id: Optional[str] = None,
        app_id: Optional[str] = None,
        service_id: Optional[str] = None,
        cursor: OptionalNullable[str] = UNSET,
        limit: Optional[int] = 20,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Optional[models.AccountingQuotesAllResponse]:
        r"""List Quotes

        List Quotes

        :param raw: Include raw response. Mostly used for debugging purposes
        :param consumer_id: ID of the consumer which you want to get or push data from
        :param app_id: The ID of your Unify application
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param cursor: Cursor to start from. You can find cursors for next/previous pages in the meta.cursors property of the response.
        :param limit: Number of results to return. Minimum 1, Maximum 200, Default 20
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.AccountingQuotesAllRequest(
            raw=raw,
            consumer_id=consumer_id,
            app_id=app_id,
            service_id=service_id,
            cursor=cursor,
            limit=limit,
        )

        req = self._build_request(
            method="GET",
            path="/accounting/quotes",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.AccountingQuotesAllGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="accounting.quotesAll",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        def next_func() -> Optional[models.AccountingQuotesAllResponse]:
            body = utils.unmarshal_json(http_res.text, Union[Dict[Any, Any], List[Any]])
            next_cursor = JSONPath("$.meta.cursors.next").parse(body)

            if len(next_cursor) == 0:
                return None

            next_cursor = next_cursor[0]
            if next_cursor is None:
                return None

            return self.list(
                raw=raw,
                consumer_id=consumer_id,
                app_id=app_id,
                service_id=service_id,
                cursor=next_cursor,
                limit=limit,
                retries=retries,
            )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.AccountingQuotesAllResponse(
                get_quotes_response=unmarshal_json_response(
                    Optional[models.GetQuotesResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
                next=next_func,
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.BadRequestResponseData, http_res
            )
            raise models.BadRequestResponse(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.UnauthorizedResponseData, http_res
            )
            raise models.UnauthorizedResponse(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                models.PaymentRequiredResponseData, http_res
            )
            raise models.PaymentRequiredResponse(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                models.NotFoundResponseData, http_res
            )
            raise models.NotFoundResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.UnprocessableResponseData, http_res
            )
            raise models.UnprocessableResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "default", "application/json"):
            return models.AccountingQuotesAllResponse(
                unexpected_error_response=unmarshal_json_response(
                    Optional[models.UnexpectedErrorResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
                next=next_func,
            )

        raise models.APIError("Unexpected response received", http_res)

    async def list_async(
        self,
        *,
        raw: Optional[bool] = False,
        consumer_id: Optional[str] = None,
        app_id: Optional[str] = None,
        service_id: Optional[str] = None,
        cursor: OptionalNullable[str] = UNSET,
        limit: Optional[int] = 20,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Optional[models.AccountingQuotesAllResponse]:
        r"""List Quotes

        List Quotes

        :param raw: Include raw response. Mostly used for debugging purposes
        :param consumer_id: ID of the consumer which you want to get or push data from
        :param app_id: The ID of your Unify application
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param cursor: Cursor to start from. You can find cursors for next/previous pages in the meta.cursors property of the response.
        :param limit: Number of results to return. Minimum 1, Maximum 200, Default 20
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.AccountingQuotesAllRequest(
            raw=raw,
            consumer_id=consumer_id,
            app_id=app_id,
            service_id=service_id,
            cursor=cursor,
            limit=limit,
        )

        req = self._build_request_async(
            method="GET",
            path="/accounting/quotes",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.AccountingQuotesAllGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="accounting.quotesAll",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        def next_func() -> Optional[models.AccountingQuotesAllResponse]:
            body = utils.unmarshal_json(http_res.text, Union[Dict[Any, Any], List[Any]])
            next_cursor = JSONPath("$.meta.cursors.next").parse(body)

            if len(next_cursor) == 0:
                return None

            next_cursor = next_cursor[0]
            if next_cursor is None:
                return None

            return self.list(
                raw=raw,
                consumer_id=consumer_id,
                app_id=app_id,
                service_id=service_id,
                cursor=next_cursor,
                limit=limit,
                retries=retries,
            )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.AccountingQuotesAllResponse(
                get_quotes_response=unmarshal_json_response(
                    Optional[models.GetQuotesResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
                next=next_func,
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.BadRequestResponseData, http_res
            )
            raise models.BadRequestResponse(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.UnauthorizedResponseData, http_res
            )
            raise models.UnauthorizedResponse(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                models.PaymentRequiredResponseData, http_res
            )
            raise models.PaymentRequiredResponse(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                models.NotFoundResponseData, http_res
            )
            raise models.NotFoundResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.UnprocessableResponseData, http_res
            )
            raise models.UnprocessableResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "default", "application/json"):
            return models.AccountingQuotesAllResponse(
                unexpected_error_response=unmarshal_json_response(
                    Optional[models.UnexpectedErrorResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
                next=next_func,
            )

        raise models.APIError("Unexpected response received", http_res)

    def create(
        self,
        *,
        raw: Optional[bool] = False,
        consumer_id: Optional[str] = None,
        app_id: Optional[str] = None,
        service_id: Optional[str] = None,
        number: OptionalNullable[str] = UNSET,
        customer: OptionalNullable[
            Union[models.LinkedCustomerInput, models.LinkedCustomerInputTypedDict]
        ] = UNSET,
        sales_order_id: OptionalNullable[str] = UNSET,
        company_id: OptionalNullable[str] = UNSET,
        department_id: OptionalNullable[str] = UNSET,
        project_id: Optional[str] = None,
        quote_date: OptionalNullable[date] = UNSET,
        expiry_date: OptionalNullable[date] = UNSET,
        terms: OptionalNullable[str] = UNSET,
        reference: OptionalNullable[str] = UNSET,
        status: OptionalNullable[models.QuoteStatus] = UNSET,
        currency: OptionalNullable[models.Currency] = UNSET,
        currency_rate: OptionalNullable[float] = UNSET,
        tax_inclusive: OptionalNullable[bool] = UNSET,
        sub_total: OptionalNullable[float] = UNSET,
        total_tax: OptionalNullable[float] = UNSET,
        tax_code: OptionalNullable[str] = UNSET,
        discount_percentage: OptionalNullable[float] = UNSET,
        discount_amount: OptionalNullable[float] = UNSET,
        total: OptionalNullable[float] = UNSET,
        customer_memo: OptionalNullable[str] = UNSET,
        line_items: Optional[
            Union[
                List[models.QuoteLineItemInput],
                List[models.QuoteLineItemInputTypedDict],
            ]
        ] = None,
        billing_address: Optional[
            Union[models.Address, models.AddressTypedDict]
        ] = None,
        shipping_address: Optional[
            Union[models.Address, models.AddressTypedDict]
        ] = None,
        tracking_categories: OptionalNullable[
            Union[
                List[Nullable[models.LinkedTrackingCategory]],
                List[Nullable[models.LinkedTrackingCategoryTypedDict]],
            ]
        ] = UNSET,
        template_id: OptionalNullable[str] = UNSET,
        source_document_url: OptionalNullable[str] = UNSET,
        custom_fields: Optional[
            Union[List[models.CustomField], List[models.CustomFieldTypedDict]]
        ] = None,
        row_version: OptionalNullable[str] = UNSET,
        pass_through: Optional[
            Union[List[models.PassThroughBody], List[models.PassThroughBodyTypedDict]]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.AccountingQuotesAddResponse:
        r"""Create Quote

        Create Quote

        :param raw: Include raw response. Mostly used for debugging purposes
        :param consumer_id: ID of the consumer which you want to get or push data from
        :param app_id: The ID of your Unify application
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param number: Quote number.
        :param customer: The customer this entity is linked to.
        :param sales_order_id: The unique identifier for the sales order.
        :param company_id: The company ID the transaction belongs to
        :param department_id: The ID of the department
        :param project_id: The unique identifier for the linked project.
        :param quote_date: Date quote was issued - YYYY-MM-DD.
        :param expiry_date: The date until which the quote is valid - YYYY-MM-DD.
        :param terms: Terms of the quote.
        :param reference: Optional reference identifier for the transaction.
        :param status: Quote status
        :param currency: Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217).
        :param currency_rate: Currency Exchange Rate at the time entity was recorded/generated.
        :param tax_inclusive: Amounts are including tax
        :param sub_total: Sub-total amount, normally before tax.
        :param total_tax: Total tax amount applied to this quote.
        :param tax_code: Applicable tax id/code override if tax is not supplied on a line item basis.
        :param discount_percentage: Discount percentage applied to this quote.
        :param discount_amount: Discount amount applied to this quote.
        :param total: Total amount of quote, including tax.
        :param customer_memo: Customer memo
        :param line_items:
        :param billing_address:
        :param shipping_address:
        :param tracking_categories: A list of linked tracking categories.
        :param template_id: Optional quote template
        :param source_document_url: URL link to a source document - shown as 'Go to [appName]' in the downstream app.
        :param custom_fields:
        :param row_version: A binary value used to detect updates to a object and prevent data conflicts. It is incremented each time an update is made to the object.
        :param pass_through: The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.AccountingQuotesAddRequest(
            raw=raw,
            consumer_id=consumer_id,
            app_id=app_id,
            service_id=service_id,
            quote=models.QuoteInput(
                number=number,
                customer=utils.get_pydantic_model(
                    customer, OptionalNullable[models.LinkedCustomerInput]
                ),
                sales_order_id=sales_order_id,
                company_id=company_id,
                department_id=department_id,
                project_id=project_id,
                quote_date=quote_date,
                expiry_date=expiry_date,
                terms=terms,
                reference=reference,
                status=status,
                currency=currency,
                currency_rate=currency_rate,
                tax_inclusive=tax_inclusive,
                sub_total=sub_total,
                total_tax=total_tax,
                tax_code=tax_code,
                discount_percentage=discount_percentage,
                discount_amount=discount_amount,
                total=total,
                customer_memo=customer_memo,
                line_items=utils.get_pydantic_model(
                    line_items, Optional[List[models.QuoteLineItemInput]]
                ),
                billing_address=utils.get_pydantic_model(
                    billing_address, Optional[models.Address]
                ),
                shipping_address=utils.get_pydantic_model(
                    shipping_address, Optional[models.Address]
                ),
                tracking_categories=utils.get_pydantic_model(
                    tracking_categories,
                    OptionalNullable[List[Nullable[models.LinkedTrackingCategory]]],
                ),
                template_id=template_id,
                source_document_url=source_document_url,
                custom_fields=utils.get_pydantic_model(
                    custom_fields, Optional[List[models.CustomField]]
                ),
                row_version=row_version,
                pass_through=utils.get_pydantic_model(
                    pass_through, Optional[List[models.PassThroughBody]]
                ),
            ),
        )

        req = self._build_request(
            method="POST",
            path="/accounting/quotes",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.AccountingQuotesAddGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.quote, False, False, "json", models.QuoteInput
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="accounting.quotesAdd",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return models.AccountingQuotesAddResponse(
                create_quote_response=unmarshal_json_response(
                    Optional[models.CreateQuoteResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.BadRequestResponseData, http_res
            )
            raise models.BadRequestResponse(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.UnauthorizedResponseData, http_res
            )
            raise models.UnauthorizedResponse(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                models.PaymentRequiredResponseData, http_res
            )
            raise models.PaymentRequiredResponse(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                models.NotFoundResponseData, http_res
            )
            raise models.NotFoundResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.UnprocessableResponseData, http_res
            )
            raise models.UnprocessableResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "default", "application/json"):
            return models.AccountingQuotesAddResponse(
                unexpected_error_response=unmarshal_json_response(
                    Optional[models.UnexpectedErrorResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )

        raise models.APIError("Unexpected response received", http_res)

    async def create_async(
        self,
        *,
        raw: Optional[bool] = False,
        consumer_id: Optional[str] = None,
        app_id: Optional[str] = None,
        service_id: Optional[str] = None,
        number: OptionalNullable[str] = UNSET,
        customer: OptionalNullable[
            Union[models.LinkedCustomerInput, models.LinkedCustomerInputTypedDict]
        ] = UNSET,
        sales_order_id: OptionalNullable[str] = UNSET,
        company_id: OptionalNullable[str] = UNSET,
        department_id: OptionalNullable[str] = UNSET,
        project_id: Optional[str] = None,
        quote_date: OptionalNullable[date] = UNSET,
        expiry_date: OptionalNullable[date] = UNSET,
        terms: OptionalNullable[str] = UNSET,
        reference: OptionalNullable[str] = UNSET,
        status: OptionalNullable[models.QuoteStatus] = UNSET,
        currency: OptionalNullable[models.Currency] = UNSET,
        currency_rate: OptionalNullable[float] = UNSET,
        tax_inclusive: OptionalNullable[bool] = UNSET,
        sub_total: OptionalNullable[float] = UNSET,
        total_tax: OptionalNullable[float] = UNSET,
        tax_code: OptionalNullable[str] = UNSET,
        discount_percentage: OptionalNullable[float] = UNSET,
        discount_amount: OptionalNullable[float] = UNSET,
        total: OptionalNullable[float] = UNSET,
        customer_memo: OptionalNullable[str] = UNSET,
        line_items: Optional[
            Union[
                List[models.QuoteLineItemInput],
                List[models.QuoteLineItemInputTypedDict],
            ]
        ] = None,
        billing_address: Optional[
            Union[models.Address, models.AddressTypedDict]
        ] = None,
        shipping_address: Optional[
            Union[models.Address, models.AddressTypedDict]
        ] = None,
        tracking_categories: OptionalNullable[
            Union[
                List[Nullable[models.LinkedTrackingCategory]],
                List[Nullable[models.LinkedTrackingCategoryTypedDict]],
            ]
        ] = UNSET,
        template_id: OptionalNullable[str] = UNSET,
        source_document_url: OptionalNullable[str] = UNSET,
        custom_fields: Optional[
            Union[List[models.CustomField], List[models.CustomFieldTypedDict]]
        ] = None,
        row_version: OptionalNullable[str] = UNSET,
        pass_through: Optional[
            Union[List[models.PassThroughBody], List[models.PassThroughBodyTypedDict]]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.AccountingQuotesAddResponse:
        r"""Create Quote

        Create Quote

        :param raw: Include raw response. Mostly used for debugging purposes
        :param consumer_id: ID of the consumer which you want to get or push data from
        :param app_id: The ID of your Unify application
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param number: Quote number.
        :param customer: The customer this entity is linked to.
        :param sales_order_id: The unique identifier for the sales order.
        :param company_id: The company ID the transaction belongs to
        :param department_id: The ID of the department
        :param project_id: The unique identifier for the linked project.
        :param quote_date: Date quote was issued - YYYY-MM-DD.
        :param expiry_date: The date until which the quote is valid - YYYY-MM-DD.
        :param terms: Terms of the quote.
        :param reference: Optional reference identifier for the transaction.
        :param status: Quote status
        :param currency: Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217).
        :param currency_rate: Currency Exchange Rate at the time entity was recorded/generated.
        :param tax_inclusive: Amounts are including tax
        :param sub_total: Sub-total amount, normally before tax.
        :param total_tax: Total tax amount applied to this quote.
        :param tax_code: Applicable tax id/code override if tax is not supplied on a line item basis.
        :param discount_percentage: Discount percentage applied to this quote.
        :param discount_amount: Discount amount applied to this quote.
        :param total: Total amount of quote, including tax.
        :param customer_memo: Customer memo
        :param line_items:
        :param billing_address:
        :param shipping_address:
        :param tracking_categories: A list of linked tracking categories.
        :param template_id: Optional quote template
        :param source_document_url: URL link to a source document - shown as 'Go to [appName]' in the downstream app.
        :param custom_fields:
        :param row_version: A binary value used to detect updates to a object and prevent data conflicts. It is incremented each time an update is made to the object.
        :param pass_through: The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.AccountingQuotesAddRequest(
            raw=raw,
            consumer_id=consumer_id,
            app_id=app_id,
            service_id=service_id,
            quote=models.QuoteInput(
                number=number,
                customer=utils.get_pydantic_model(
                    customer, OptionalNullable[models.LinkedCustomerInput]
                ),
                sales_order_id=sales_order_id,
                company_id=company_id,
                department_id=department_id,
                project_id=project_id,
                quote_date=quote_date,
                expiry_date=expiry_date,
                terms=terms,
                reference=reference,
                status=status,
                currency=currency,
                currency_rate=currency_rate,
                tax_inclusive=tax_inclusive,
                sub_total=sub_total,
                total_tax=total_tax,
                tax_code=tax_code,
                discount_percentage=discount_percentage,
                discount_amount=discount_amount,
                total=total,
                customer_memo=customer_memo,
                line_items=utils.get_pydantic_model(
                    line_items, Optional[List[models.QuoteLineItemInput]]
                ),
                billing_address=utils.get_pydantic_model(
                    billing_address, Optional[models.Address]
                ),
                shipping_address=utils.get_pydantic_model(
                    shipping_address, Optional[models.Address]
                ),
                tracking_categories=utils.get_pydantic_model(
                    tracking_categories,
                    OptionalNullable[List[Nullable[models.LinkedTrackingCategory]]],
                ),
                template_id=template_id,
                source_document_url=source_document_url,
                custom_fields=utils.get_pydantic_model(
                    custom_fields, Optional[List[models.CustomField]]
                ),
                row_version=row_version,
                pass_through=utils.get_pydantic_model(
                    pass_through, Optional[List[models.PassThroughBody]]
                ),
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/accounting/quotes",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.AccountingQuotesAddGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.quote, False, False, "json", models.QuoteInput
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="accounting.quotesAdd",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return models.AccountingQuotesAddResponse(
                create_quote_response=unmarshal_json_response(
                    Optional[models.CreateQuoteResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.BadRequestResponseData, http_res
            )
            raise models.BadRequestResponse(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.UnauthorizedResponseData, http_res
            )
            raise models.UnauthorizedResponse(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                models.PaymentRequiredResponseData, http_res
            )
            raise models.PaymentRequiredResponse(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                models.NotFoundResponseData, http_res
            )
            raise models.NotFoundResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.UnprocessableResponseData, http_res
            )
            raise models.UnprocessableResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "default", "application/json"):
            return models.AccountingQuotesAddResponse(
                unexpected_error_response=unmarshal_json_response(
                    Optional[models.UnexpectedErrorResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )

        raise models.APIError("Unexpected response received", http_res)

    def get(
        self,
        *,
        id: str,
        consumer_id: Optional[str] = None,
        app_id: Optional[str] = None,
        service_id: Optional[str] = None,
        raw: Optional[bool] = False,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.AccountingQuotesOneResponse:
        r"""Get Quote

        Get Quote

        :param id: ID of the record you are acting upon.
        :param consumer_id: ID of the consumer which you want to get or push data from
        :param app_id: The ID of your Unify application
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param raw: Include raw response. Mostly used for debugging purposes
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.AccountingQuotesOneRequest(
            id=id,
            consumer_id=consumer_id,
            app_id=app_id,
            service_id=service_id,
            raw=raw,
        )

        req = self._build_request(
            method="GET",
            path="/accounting/quotes/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.AccountingQuotesOneGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="accounting.quotesOne",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.AccountingQuotesOneResponse(
                get_quote_response=unmarshal_json_response(
                    Optional[models.GetQuoteResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.BadRequestResponseData, http_res
            )
            raise models.BadRequestResponse(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.UnauthorizedResponseData, http_res
            )
            raise models.UnauthorizedResponse(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                models.PaymentRequiredResponseData, http_res
            )
            raise models.PaymentRequiredResponse(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                models.NotFoundResponseData, http_res
            )
            raise models.NotFoundResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.UnprocessableResponseData, http_res
            )
            raise models.UnprocessableResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "default", "application/json"):
            return models.AccountingQuotesOneResponse(
                unexpected_error_response=unmarshal_json_response(
                    Optional[models.UnexpectedErrorResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )

        raise models.APIError("Unexpected response received", http_res)

    async def get_async(
        self,
        *,
        id: str,
        consumer_id: Optional[str] = None,
        app_id: Optional[str] = None,
        service_id: Optional[str] = None,
        raw: Optional[bool] = False,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.AccountingQuotesOneResponse:
        r"""Get Quote

        Get Quote

        :param id: ID of the record you are acting upon.
        :param consumer_id: ID of the consumer which you want to get or push data from
        :param app_id: The ID of your Unify application
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param raw: Include raw response. Mostly used for debugging purposes
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.AccountingQuotesOneRequest(
            id=id,
            consumer_id=consumer_id,
            app_id=app_id,
            service_id=service_id,
            raw=raw,
        )

        req = self._build_request_async(
            method="GET",
            path="/accounting/quotes/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.AccountingQuotesOneGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="accounting.quotesOne",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.AccountingQuotesOneResponse(
                get_quote_response=unmarshal_json_response(
                    Optional[models.GetQuoteResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.BadRequestResponseData, http_res
            )
            raise models.BadRequestResponse(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.UnauthorizedResponseData, http_res
            )
            raise models.UnauthorizedResponse(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                models.PaymentRequiredResponseData, http_res
            )
            raise models.PaymentRequiredResponse(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                models.NotFoundResponseData, http_res
            )
            raise models.NotFoundResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.UnprocessableResponseData, http_res
            )
            raise models.UnprocessableResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "default", "application/json"):
            return models.AccountingQuotesOneResponse(
                unexpected_error_response=unmarshal_json_response(
                    Optional[models.UnexpectedErrorResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )

        raise models.APIError("Unexpected response received", http_res)

    def update(
        self,
        *,
        id: str,
        consumer_id: Optional[str] = None,
        app_id: Optional[str] = None,
        service_id: Optional[str] = None,
        raw: Optional[bool] = False,
        number: OptionalNullable[str] = UNSET,
        customer: OptionalNullable[
            Union[models.LinkedCustomerInput, models.LinkedCustomerInputTypedDict]
        ] = UNSET,
        sales_order_id: OptionalNullable[str] = UNSET,
        company_id: OptionalNullable[str] = UNSET,
        department_id: OptionalNullable[str] = UNSET,
        project_id: Optional[str] = None,
        quote_date: OptionalNullable[date] = UNSET,
        expiry_date: OptionalNullable[date] = UNSET,
        terms: OptionalNullable[str] = UNSET,
        reference: OptionalNullable[str] = UNSET,
        status: OptionalNullable[models.QuoteStatus] = UNSET,
        currency: OptionalNullable[models.Currency] = UNSET,
        currency_rate: OptionalNullable[float] = UNSET,
        tax_inclusive: OptionalNullable[bool] = UNSET,
        sub_total: OptionalNullable[float] = UNSET,
        total_tax: OptionalNullable[float] = UNSET,
        tax_code: OptionalNullable[str] = UNSET,
        discount_percentage: OptionalNullable[float] = UNSET,
        discount_amount: OptionalNullable[float] = UNSET,
        total: OptionalNullable[float] = UNSET,
        customer_memo: OptionalNullable[str] = UNSET,
        line_items: Optional[
            Union[
                List[models.QuoteLineItemInput],
                List[models.QuoteLineItemInputTypedDict],
            ]
        ] = None,
        billing_address: Optional[
            Union[models.Address, models.AddressTypedDict]
        ] = None,
        shipping_address: Optional[
            Union[models.Address, models.AddressTypedDict]
        ] = None,
        tracking_categories: OptionalNullable[
            Union[
                List[Nullable[models.LinkedTrackingCategory]],
                List[Nullable[models.LinkedTrackingCategoryTypedDict]],
            ]
        ] = UNSET,
        template_id: OptionalNullable[str] = UNSET,
        source_document_url: OptionalNullable[str] = UNSET,
        custom_fields: Optional[
            Union[List[models.CustomField], List[models.CustomFieldTypedDict]]
        ] = None,
        row_version: OptionalNullable[str] = UNSET,
        pass_through: Optional[
            Union[List[models.PassThroughBody], List[models.PassThroughBodyTypedDict]]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.AccountingQuotesUpdateResponse:
        r"""Update Quote

        Update Quote

        :param id: ID of the record you are acting upon.
        :param consumer_id: ID of the consumer which you want to get or push data from
        :param app_id: The ID of your Unify application
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param raw: Include raw response. Mostly used for debugging purposes
        :param number: Quote number.
        :param customer: The customer this entity is linked to.
        :param sales_order_id: The unique identifier for the sales order.
        :param company_id: The company ID the transaction belongs to
        :param department_id: The ID of the department
        :param project_id: The unique identifier for the linked project.
        :param quote_date: Date quote was issued - YYYY-MM-DD.
        :param expiry_date: The date until which the quote is valid - YYYY-MM-DD.
        :param terms: Terms of the quote.
        :param reference: Optional reference identifier for the transaction.
        :param status: Quote status
        :param currency: Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217).
        :param currency_rate: Currency Exchange Rate at the time entity was recorded/generated.
        :param tax_inclusive: Amounts are including tax
        :param sub_total: Sub-total amount, normally before tax.
        :param total_tax: Total tax amount applied to this quote.
        :param tax_code: Applicable tax id/code override if tax is not supplied on a line item basis.
        :param discount_percentage: Discount percentage applied to this quote.
        :param discount_amount: Discount amount applied to this quote.
        :param total: Total amount of quote, including tax.
        :param customer_memo: Customer memo
        :param line_items:
        :param billing_address:
        :param shipping_address:
        :param tracking_categories: A list of linked tracking categories.
        :param template_id: Optional quote template
        :param source_document_url: URL link to a source document - shown as 'Go to [appName]' in the downstream app.
        :param custom_fields:
        :param row_version: A binary value used to detect updates to a object and prevent data conflicts. It is incremented each time an update is made to the object.
        :param pass_through: The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.AccountingQuotesUpdateRequest(
            id=id,
            consumer_id=consumer_id,
            app_id=app_id,
            service_id=service_id,
            raw=raw,
            quote=models.QuoteInput(
                number=number,
                customer=utils.get_pydantic_model(
                    customer, OptionalNullable[models.LinkedCustomerInput]
                ),
                sales_order_id=sales_order_id,
                company_id=company_id,
                department_id=department_id,
                project_id=project_id,
                quote_date=quote_date,
                expiry_date=expiry_date,
                terms=terms,
                reference=reference,
                status=status,
                currency=currency,
                currency_rate=currency_rate,
                tax_inclusive=tax_inclusive,
                sub_total=sub_total,
                total_tax=total_tax,
                tax_code=tax_code,
                discount_percentage=discount_percentage,
                discount_amount=discount_amount,
                total=total,
                customer_memo=customer_memo,
                line_items=utils.get_pydantic_model(
                    line_items, Optional[List[models.QuoteLineItemInput]]
                ),
                billing_address=utils.get_pydantic_model(
                    billing_address, Optional[models.Address]
                ),
                shipping_address=utils.get_pydantic_model(
                    shipping_address, Optional[models.Address]
                ),
                tracking_categories=utils.get_pydantic_model(
                    tracking_categories,
                    OptionalNullable[List[Nullable[models.LinkedTrackingCategory]]],
                ),
                template_id=template_id,
                source_document_url=source_document_url,
                custom_fields=utils.get_pydantic_model(
                    custom_fields, Optional[List[models.CustomField]]
                ),
                row_version=row_version,
                pass_through=utils.get_pydantic_model(
                    pass_through, Optional[List[models.PassThroughBody]]
                ),
            ),
        )

        req = self._build_request(
            method="PATCH",
            path="/accounting/quotes/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.AccountingQuotesUpdateGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.quote, False, False, "json", models.QuoteInput
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="accounting.quotesUpdate",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.AccountingQuotesUpdateResponse(
                update_quote_response=unmarshal_json_response(
                    Optional[models.UpdateQuoteResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.BadRequestResponseData, http_res
            )
            raise models.BadRequestResponse(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.UnauthorizedResponseData, http_res
            )
            raise models.UnauthorizedResponse(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                models.PaymentRequiredResponseData, http_res
            )
            raise models.PaymentRequiredResponse(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                models.NotFoundResponseData, http_res
            )
            raise models.NotFoundResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.UnprocessableResponseData, http_res
            )
            raise models.UnprocessableResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "default", "application/json"):
            return models.AccountingQuotesUpdateResponse(
                unexpected_error_response=unmarshal_json_response(
                    Optional[models.UnexpectedErrorResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )

        raise models.APIError("Unexpected response received", http_res)

    async def update_async(
        self,
        *,
        id: str,
        consumer_id: Optional[str] = None,
        app_id: Optional[str] = None,
        service_id: Optional[str] = None,
        raw: Optional[bool] = False,
        number: OptionalNullable[str] = UNSET,
        customer: OptionalNullable[
            Union[models.LinkedCustomerInput, models.LinkedCustomerInputTypedDict]
        ] = UNSET,
        sales_order_id: OptionalNullable[str] = UNSET,
        company_id: OptionalNullable[str] = UNSET,
        department_id: OptionalNullable[str] = UNSET,
        project_id: Optional[str] = None,
        quote_date: OptionalNullable[date] = UNSET,
        expiry_date: OptionalNullable[date] = UNSET,
        terms: OptionalNullable[str] = UNSET,
        reference: OptionalNullable[str] = UNSET,
        status: OptionalNullable[models.QuoteStatus] = UNSET,
        currency: OptionalNullable[models.Currency] = UNSET,
        currency_rate: OptionalNullable[float] = UNSET,
        tax_inclusive: OptionalNullable[bool] = UNSET,
        sub_total: OptionalNullable[float] = UNSET,
        total_tax: OptionalNullable[float] = UNSET,
        tax_code: OptionalNullable[str] = UNSET,
        discount_percentage: OptionalNullable[float] = UNSET,
        discount_amount: OptionalNullable[float] = UNSET,
        total: OptionalNullable[float] = UNSET,
        customer_memo: OptionalNullable[str] = UNSET,
        line_items: Optional[
            Union[
                List[models.QuoteLineItemInput],
                List[models.QuoteLineItemInputTypedDict],
            ]
        ] = None,
        billing_address: Optional[
            Union[models.Address, models.AddressTypedDict]
        ] = None,
        shipping_address: Optional[
            Union[models.Address, models.AddressTypedDict]
        ] = None,
        tracking_categories: OptionalNullable[
            Union[
                List[Nullable[models.LinkedTrackingCategory]],
                List[Nullable[models.LinkedTrackingCategoryTypedDict]],
            ]
        ] = UNSET,
        template_id: OptionalNullable[str] = UNSET,
        source_document_url: OptionalNullable[str] = UNSET,
        custom_fields: Optional[
            Union[List[models.CustomField], List[models.CustomFieldTypedDict]]
        ] = None,
        row_version: OptionalNullable[str] = UNSET,
        pass_through: Optional[
            Union[List[models.PassThroughBody], List[models.PassThroughBodyTypedDict]]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.AccountingQuotesUpdateResponse:
        r"""Update Quote

        Update Quote

        :param id: ID of the record you are acting upon.
        :param consumer_id: ID of the consumer which you want to get or push data from
        :param app_id: The ID of your Unify application
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param raw: Include raw response. Mostly used for debugging purposes
        :param number: Quote number.
        :param customer: The customer this entity is linked to.
        :param sales_order_id: The unique identifier for the sales order.
        :param company_id: The company ID the transaction belongs to
        :param department_id: The ID of the department
        :param project_id: The unique identifier for the linked project.
        :param quote_date: Date quote was issued - YYYY-MM-DD.
        :param expiry_date: The date until which the quote is valid - YYYY-MM-DD.
        :param terms: Terms of the quote.
        :param reference: Optional reference identifier for the transaction.
        :param status: Quote status
        :param currency: Indicates the associated currency for an amount of money. Values correspond to [ISO 4217](https://en.wikipedia.org/wiki/ISO_4217).
        :param currency_rate: Currency Exchange Rate at the time entity was recorded/generated.
        :param tax_inclusive: Amounts are including tax
        :param sub_total: Sub-total amount, normally before tax.
        :param total_tax: Total tax amount applied to this quote.
        :param tax_code: Applicable tax id/code override if tax is not supplied on a line item basis.
        :param discount_percentage: Discount percentage applied to this quote.
        :param discount_amount: Discount amount applied to this quote.
        :param total: Total amount of quote, including tax.
        :param customer_memo: Customer memo
        :param line_items:
        :param billing_address:
        :param shipping_address:
        :param tracking_categories: A list of linked tracking categories.
        :param template_id: Optional quote template
        :param source_document_url: URL link to a source document - shown as 'Go to [appName]' in the downstream app.
        :param custom_fields:
        :param row_version: A binary value used to detect updates to a object and prevent data conflicts. It is incremented each time an update is made to the object.
        :param pass_through: The pass_through property allows passing service-specific, custom data or structured modifications in request body when creating or updating resources.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.AccountingQuotesUpdateRequest(
            id=id,
            consumer_id=consumer_id,
            app_id=app_id,
            service_id=service_id,
            raw=raw,
            quote=models.QuoteInput(
                number=number,
                customer=utils.get_pydantic_model(
                    customer, OptionalNullable[models.LinkedCustomerInput]
                ),
                sales_order_id=sales_order_id,
                company_id=company_id,
                department_id=department_id,
                project_id=project_id,
                quote_date=quote_date,
                expiry_date=expiry_date,
                terms=terms,
                reference=reference,
                status=status,
                currency=currency,
                currency_rate=currency_rate,
                tax_inclusive=tax_inclusive,
                sub_total=sub_total,
                total_tax=total_tax,
                tax_code=tax_code,
                discount_percentage=discount_percentage,
                discount_amount=discount_amount,
                total=total,
                customer_memo=customer_memo,
                line_items=utils.get_pydantic_model(
                    line_items, Optional[List[models.QuoteLineItemInput]]
                ),
                billing_address=utils.get_pydantic_model(
                    billing_address, Optional[models.Address]
                ),
                shipping_address=utils.get_pydantic_model(
                    shipping_address, Optional[models.Address]
                ),
                tracking_categories=utils.get_pydantic_model(
                    tracking_categories,
                    OptionalNullable[List[Nullable[models.LinkedTrackingCategory]]],
                ),
                template_id=template_id,
                source_document_url=source_document_url,
                custom_fields=utils.get_pydantic_model(
                    custom_fields, Optional[List[models.CustomField]]
                ),
                row_version=row_version,
                pass_through=utils.get_pydantic_model(
                    pass_through, Optional[List[models.PassThroughBody]]
                ),
            ),
        )

        req = self._build_request_async(
            method="PATCH",
            path="/accounting/quotes/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.AccountingQuotesUpdateGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.quote, False, False, "json", models.QuoteInput
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="accounting.quotesUpdate",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.AccountingQuotesUpdateResponse(
                update_quote_response=unmarshal_json_response(
                    Optional[models.UpdateQuoteResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.BadRequestResponseData, http_res
            )
            raise models.BadRequestResponse(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.UnauthorizedResponseData, http_res
            )
            raise models.UnauthorizedResponse(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                models.PaymentRequiredResponseData, http_res
            )
            raise models.PaymentRequiredResponse(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                models.NotFoundResponseData, http_res
            )
            raise models.NotFoundResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.UnprocessableResponseData, http_res
            )
            raise models.UnprocessableResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "default", "application/json"):
            return models.AccountingQuotesUpdateResponse(
                unexpected_error_response=unmarshal_json_response(
                    Optional[models.UnexpectedErrorResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )

        raise models.APIError("Unexpected response received", http_res)

    def delete(
        self,
        *,
        id: str,
        consumer_id: Optional[str] = None,
        app_id: Optional[str] = None,
        service_id: Optional[str] = None,
        raw: Optional[bool] = False,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.AccountingQuotesDeleteResponse:
        r"""Delete Quote

        Delete Quote

        :param id: ID of the record you are acting upon.
        :param consumer_id: ID of the consumer which you want to get or push data from
        :param app_id: The ID of your Unify application
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param raw: Include raw response. Mostly used for debugging purposes
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.AccountingQuotesDeleteRequest(
            id=id,
            consumer_id=consumer_id,
            app_id=app_id,
            service_id=service_id,
            raw=raw,
        )

        req = self._build_request(
            method="DELETE",
            path="/accounting/quotes/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.AccountingQuotesDeleteGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="accounting.quotesDelete",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.AccountingQuotesDeleteResponse(
                delete_quote_response=unmarshal_json_response(
                    Optional[models.DeleteQuoteResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.BadRequestResponseData, http_res
            )
            raise models.BadRequestResponse(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.UnauthorizedResponseData, http_res
            )
            raise models.UnauthorizedResponse(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                models.PaymentRequiredResponseData, http_res
            )
            raise models.PaymentRequiredResponse(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                models.NotFoundResponseData, http_res
            )
            raise models.NotFoundResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.UnprocessableResponseData, http_res
            )
            raise models.UnprocessableResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "default", "application/json"):
            return models.AccountingQuotesDeleteResponse(
                unexpected_error_response=unmarshal_json_response(
                    Optional[models.UnexpectedErrorResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )

        raise models.APIError("Unexpected response received", http_res)

    async def delete_async(
        self,
        *,
        id: str,
        consumer_id: Optional[str] = None,
        app_id: Optional[str] = None,
        service_id: Optional[str] = None,
        raw: Optional[bool] = False,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.AccountingQuotesDeleteResponse:
        r"""Delete Quote

        Delete Quote

        :param id: ID of the record you are acting upon.
        :param consumer_id: ID of the consumer which you want to get or push data from
        :param app_id: The ID of your Unify application
        :param service_id: Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API.
        :param raw: Include raw response. Mostly used for debugging purposes
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.AccountingQuotesDeleteRequest(
            id=id,
            consumer_id=consumer_id,
            app_id=app_id,
            service_id=service_id,
            raw=raw,
        )

        req = self._build_request_async(
            method="DELETE",
            path="/accounting/quotes/{id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.AccountingQuotesDeleteGlobals(
                consumer_id=self.sdk_configuration.globals.consumer_id,
                app_id=self.sdk_configuration.globals.app_id,
            ),
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 60000, 1.5, 3600000), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["408", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="accounting.quotesDelete",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "402", "404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.AccountingQuotesDeleteResponse(
                delete_quote_response=unmarshal_json_response(
                    Optional[models.DeleteQuoteResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                models.BadRequestResponseData, http_res
            )
            raise models.BadRequestResponse(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.UnauthorizedResponseData, http_res
            )
            raise models.UnauthorizedResponse(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                models.PaymentRequiredResponseData, http_res
            )
            raise models.PaymentRequiredResponse(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                models.NotFoundResponseData, http_res
            )
            raise models.NotFoundResponse(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                models.UnprocessableResponseData, http_res
            )
            raise models.UnprocessableResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "default", "application/json"):
            return models.AccountingQuotesDeleteResponse(
                unexpected_error_response=unmarshal_json_response(
                    Optional[models.UnexpectedErrorResponse], http_res
                ),
                http_meta=models.HTTPMetadata(request=req, response=http_res),
            )

        raise models.APIError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .getpurchaseordersresponse import (
    GetPurchaseOrdersResponse,
    GetPurchaseOrdersResponseTypedDict,
)
from .httpmetadata import HTTPMetadata, HTTPMetadataTypedDict
from .purchaseordersfilter import PurchaseOrdersFilter, PurchaseOrdersFilterTypedDict
from .purchaseorderssort import PurchaseOrdersSort, PurchaseOrdersSortTypedDict
from .unexpectederrorresponse import (
    UnexpectedErrorResponse,
    UnexpectedErrorResponseTypedDict,
)
from apideck_unify.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from apideck_unify.utils import FieldMetadata, HeaderMetadata, QueryParamMetadata
import pydantic
from pydantic import model_serializer
from typing import Any, Callable, Dict, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class AccountingPurchaseOrdersAllGlobalsTypedDict(TypedDict):
    consumer_id: NotRequired[str]
    r"""ID of the consumer which you want to get or push data from"""
    app_id: NotRequired[str]
    r"""The ID of your Unify application"""


class AccountingPurchaseOrdersAllGlobals(BaseModel):
    consumer_id: Annotated[
        Optional[str],
        pydantic.Field(alias="x-apideck-consumer-id"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""ID of the consumer which you want to get or push data from"""

    app_id: Annotated[
        Optional[str],
        pydantic.Field(alias="x-apideck-app-id"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The ID of your Unify application"""


class AccountingPurchaseOrdersAllRequestTypedDict(TypedDict):
    raw: NotRequired[bool]
    r"""Include raw response. Mostly used for debugging purposes"""
    consumer_id: NotRequired[str]
    r"""ID of the consumer which you want to get or push data from"""
    app_id: NotRequired[str]
    r"""The ID of your Unify application"""
    service_id: NotRequired[str]
    r"""Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API."""
    cursor: NotRequired[Nullable[str]]
    r"""Cursor to start from. You can find cursors for next/previous pages in the meta.cursors property of the response."""
    pass_through: NotRequired[Dict[str, Any]]
    r"""Optional unmapped key/values that will be passed through to downstream as query parameters. Ie: ?pass_through[search]=leads becomes ?search=leads"""
    limit: NotRequired[int]
    r"""Number of results to return. Minimum 1, Maximum 200, Default 20"""
    filter_: NotRequired[PurchaseOrdersFilterTypedDict]
    r"""Apply filters"""
    sort: NotRequired[PurchaseOrdersSortTypedDict]
    r"""Apply sorting"""


class AccountingPurchaseOrdersAllRequest(BaseModel):
    raw: Annotated[
        Optional[bool],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = False
    r"""Include raw response. Mostly used for debugging purposes"""

    consumer_id: Annotated[
        Optional[str],
        pydantic.Field(alias="x-apideck-consumer-id"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""ID of the consumer which you want to get or push data from"""

    app_id: Annotated[
        Optional[str],
        pydantic.Field(alias="x-apideck-app-id"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""The ID of your Unify application"""

    service_id: Annotated[
        Optional[str],
        pydantic.Field(alias="x-apideck-service-id"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""Provide the service id you want to call (e.g., pipedrive). Only needed when a consumer has activated multiple integrations for a Unified API."""

    cursor: Annotated[
        OptionalNullable[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = UNSET
    r"""Cursor to start from. You can find cursors for next/previous pages in the meta.cursors property of the response."""

    pass_through: Annotated[
        Optional[Dict[str, Any]],
        FieldMetadata(query=QueryParamMetadata(style="deepObject", explode=True)),
    ] = None
    r"""Optional unmapped key/values that will be passed through to downstream as query parameters. Ie: ?pass_through[search]=leads becomes ?search=leads"""

    limit: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 20
    r"""Number of results to return. Minimum 1, Maximum 200, Default 20"""

    filter_: Annotated[
        Optional[PurchaseOrdersFilter],
        pydantic.Field(alias="filter"),
        FieldMetadata(query=QueryParamMetadata(style="deepObject", explode=True)),
    ] = None
    r"""Apply filters"""

    sort: Annotated[
        Optional[PurchaseOrdersSort],
        FieldMetadata(query=QueryParamMetadata(style="deepObject", explode=True)),
    ] = None
    r"""Apply sorting"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "raw",
            "consumerId",
            "appId",
            "serviceId",
            "cursor",
            "pass_through",
            "limit",
            "filter",
            "sort",
        ]
        nullable_fields = ["cursor"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class AccountingPurchaseOrdersAllResponseTypedDict(TypedDict):
    http_meta: HTTPMetadataTypedDict
    get_purchase_orders_response: NotRequired[GetPurchaseOrdersResponseTypedDict]
    r"""PurchaseOrders"""
    unexpected_error_response: NotRequired[UnexpectedErrorResponseTypedDict]
    r"""Unexpected error"""


class AccountingPurchaseOrdersAllResponse(BaseModel):
    next: Callable[[], Optional[AccountingPurchaseOrdersAllResponse]]

    http_meta: Annotated[Optional[HTTPMetadata], pydantic.Field(exclude=True)] = None

    get_purchase_orders_response: Optional[GetPurchaseOrdersResponse] = None
    r"""PurchaseOrders"""

    unexpected_error_response: Optional[UnexpectedErrorResponse] = None
    r"""Unexpected error"""
